<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'wp_easycart_admin_cart_importer' ) ) :

	final class wp_easycart_admin_cart_importer {

		protected static $_instance = null;

		private $wpdb;

		public $oscommerce_import_file;
		public $woo_import_file;
		public $square_import_file;
		public $settings_file;

		public static function instance( ) {

			if( is_null( self::$_instance ) ) {
				self::$_instance = new self(  );
			}
			return self::$_instance;

		}

		public function __construct( ){
			// Keep reference to wpdb
			global $wpdb;
			$this->wpdb =& $wpdb;

			// Setup File Names 
			$this->oscommerce_import_file	 	= EC_PLUGIN_DIRECTORY . '/admin/template/settings/cart-importer/oscommerce-import.php';
			$this->woo_import_file	 			= EC_PLUGIN_DIRECTORY . '/admin/template/settings/cart-importer/woo-import.php';
			$this->square_import_file	 		= EC_PLUGIN_DIRECTORY . '/admin/template/settings/cart-importer/square-import.php';
			$this->settings_file		 		= EC_PLUGIN_DIRECTORY . '/admin/template/settings/cart-importer/settings.php';
			$this->shopify_import_file	 		= EC_PLUGIN_DIRECTORY . '/admin/template/settings/cart-importer/shopify-import.php';

			// Actions
			add_action( 'wpeasycart_admin_cart_importer', array( $this, 'load_woo_importer' ) );
			add_action( 'wpeasycart_admin_cart_importer', array( $this, 'load_oscommerce_importer' ) );
			add_action( 'wpeasycart_admin_cart_importer', array( $this, 'load_square_importer' ) );
			add_action( 'wpeasycart_admin_cart_importer', array( $this, 'load_shopify_importer' ) );
			add_action( 'wp_easycart_process_post_form_action', array( $this, 'process_square_import' ) );
		}

		public function load_cart_importer( ){
			include( $this->settings_file );
		}

		public function load_woo_importer( ){
			include( $this->woo_import_file );
		}
		public function load_oscommerce_importer( ){
			include( $this->oscommerce_import_file );
		}

		public function load_square_importer( ){
			include( $this->square_import_file );
		}

		public function load_shopify_importer( ){
			include( $this->shopify_import_file );
		}

		public function process_square_import( ){
			if( $_POST['ec_admin_form_action'] == 'import-square-products' ){
				$square = new ec_square( );
				$response = $square->get_catalog( );

				while( $response ){

					foreach( $response->objects as $object ){
						if( $object->type == "CATEGORY" ){
							$square->insert_category( $object );

						}else if( $object->type == "ITEM" ){
							$square->insert_product( $object );

						}
					}
					if( $response->cursor ){
						$response = $square->get_catalog( $response->cursor );
					}else{
						$response = false;
					}
				}

				header( "location:admin.php?page=wp-easycart-settings&subpage=cart-importer&ec_success=square-imported" );
				die( );
			}
		}

		public function square_import_modifiers( $cursor, $curr_count ){ // These are our option items
			$square = new ec_square( );
			$response = $square->get_modifiers( $cursor );

			foreach( $response->objects as $object ){
				$square->insert_option_item( $object, (bool) $_POST['sync_modifiers'] );
				$curr_count++;
			}
			if( $response->cursor ){
				echo json_encode( array(
					'has_more'      => true,
					'cursor'        => $response->cursor,
					'curr_count'    => $curr_count
				) );
			}else{
				echo json_encode( array(
					'has_more'      => false,
					'curr_count'    => $curr_count
				) );
			}
		}

		public function square_import_modifier_items( $cursor, $curr_count ){ // Note, these are our options
			$square = new ec_square( );
			$response = $square->get_modifier_items( $cursor );

			if ( isset( $response->objects ) ) {
				foreach( $response->objects as $object ){
					 $square->insert_option( $object, (bool) $_POST['sync_modifiers'] );
					 $curr_count++;
				}
			}
			if ( isset( $response->cursor ) && $response->cursor ) {
				echo json_encode( array(
					'has_more'      => true,
					'cursor'        => $response->cursor,
					'curr_count'    => $curr_count
				) );
			} else {
				echo json_encode( array(
					'has_more'      => false,
					'curr_count'    => $curr_count
				) );
			}
		}

		public function square_import_categories( $cursor, $curr_count ){
			$square = new ec_square( );
			$response = $square->get_catalog( false, 0, $types = array( 'CATEGORY' ) );

			if ( isset( $response->objects ) ) {
				foreach( $response->objects as $object ){
					$square->insert_category( $object );
					$curr_count++;
				}
			}
			
			echo json_encode( array(
				'has_more'      => false,
				'curr_count'    => $curr_count
			) );
		}
		
		public function square_sync_catalog_items( $cursor, $curr_count ) {
			$square = new ec_square( );
			$response = $square->get_catalog( $cursor, 0, array( 'ITEM' ) );
			
			foreach( $response->objects as $object ){
				$square->insert_product( $object );
				$curr_count++;
			}
			if ( isset( $response->cursor ) && $response->cursor ) {
				echo json_encode( array(
					'has_more'      => true,
					'cursor'        => $response->cursor,
					'curr_count'    => $curr_count
				) );
			} else {
				echo json_encode( array(
					'has_more'      => false,
					'curr_count'    => $curr_count
				) );
			}
		}
		
		public function square_sync_inventory_items( $cursor, $curr_count ) {
			$square = new ec_square( );
			$response = $square->get_inventory_results( $cursor );
			
			if ( isset( $response->counts ) ) {
				foreach( $response->counts as $object ){
					$this->update_inventory( $object );
					$curr_count++;
				}
			}
			if ( isset( $response->cursor ) && $response->cursor ) {
				echo json_encode( array(
					'has_more'      => true,
					'cursor'        => $response->cursor,
					'curr_count'    => $curr_count
				) );
			} else {
				echo json_encode( array(
					'has_more'      => false,
					'curr_count'    => $curr_count
				) );
			}
		}

		private function update_inventory( $object ) {
			global $wpdb;
			if ( 'ITEM_VARIATION' == $object->catalog_object_type ) {
				$wpdb->query( $wpdb->prepare( 'UPDATE ec_optionitemquantity SET quantity = %d WHERE square_id = %s', $object->quantity, $object->catalog_object_id ) );
				$found_optionitem = $wpdb->get_row( $wpdb->prepare( 'SELECT * FROM ec_optionitemquantity WHERE square_id = %s', $object->catalog_object_id ) );
				if ( $found_optionitem ) {
					$wpdb->query( $wpdb->prepare( 'UPDATE ec_product SET ec_product.stock_quantity = ( SELECT SUM( ec_optionitemquantity.quantity ) FROM ec_optionitemquantity WHERE ec_optionitemquantity.product_id = ec_product.product_id ) WHERE ec_product.product_id = %d', $found_optionitem->product_id ) );
				}
			} else {
				$wpdb->query( $wpdb->prepare( 'UPDATE ec_product SET stock_quantity = %d WHERE square_id = %s', $object->quantity, $object->catalog_object_id ) );
			}
		}

		public function square_import( $cursor, $curr_count ){
			$square = new ec_square( );
			$response = $square->get_catalog( $cursor );

			foreach( $response->objects as $object ){
				if( $object->type == "CATEGORY" ){
					$square->insert_category( $object, (bool) $_POST['sync_products'] );

				}else if( $object->type == "ITEM" ){
					$square->insert_product( $object, (bool) $_POST['sync_products'], (bool) $_POST['sync_inventory'] );
					$curr_count++;

				}
			}
			if( isset( $response->cursor ) && $response->cursor ){
				echo json_encode( array(
					'has_more'      => true,
					'cursor'        => $response->cursor,
					'curr_count'    => $curr_count
				) );
			} else {
				echo json_encode( array(
					'has_more'      => false,
					'curr_count'    => $curr_count
				) );
				flush_rewrite_rules();
			}
		}
	}
endif;

function wp_easycart_admin_cart_importer( ){
	return wp_easycart_admin_cart_importer::instance( );
}
wp_easycart_admin_cart_importer( );

add_action( 'wp_ajax_ec_admin_ajax_square_modifier_import', 'ec_admin_ajax_square_modifier_import' );
function ec_admin_ajax_square_modifier_import( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_import_modifiers( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
add_action( 'wp_ajax_ec_admin_ajax_square_modifier_items_import', 'ec_admin_ajax_square_modifier_items_import' );
function ec_admin_ajax_square_modifier_items_import( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_import_modifier_items( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
add_action( 'wp_ajax_ec_admin_ajax_square_categories_import', 'ec_admin_ajax_square_categories_import' );
function ec_admin_ajax_square_categories_import( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_import_categories( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
add_action( 'wp_ajax_ec_admin_ajax_square_sync_catalog', 'ec_admin_ajax_square_sync_catalog' );
function ec_admin_ajax_square_sync_catalog( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_sync_catalog_items( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
add_action( 'wp_ajax_ec_admin_ajax_square_sync_inventory', 'ec_admin_ajax_square_sync_inventory' );
function ec_admin_ajax_square_sync_inventory( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_sync_inventory_items( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
add_action( 'wp_ajax_ec_admin_ajax_square_import', 'ec_admin_ajax_square_import' );
function ec_admin_ajax_square_import( ){
	if ( ! wp_easycart_admin_verification()->verify_access( 'wp-easycart-cart-importer' ) ) {
		return false;
	}

	wp_easycart_admin_cart_importer( )->square_import( sanitize_text_field( wp_unslash( $_POST['cursor'] ) ), (int) $_POST['curr_count'] );
	die( );
}
